package implments

import (
	"LAPP_GAAS_GFrame_BACKEND/dao/base"
	dal "LAPP_GAAS_GFrame_BACKEND/dao/log"
	"LAPP_GAAS_GFrame_BACKEND/db"
	"LAPP_GAAS_GFrame_BACKEND/grmi"
	meta "LAPP_GAAS_GFrame_BACKEND/meta/log"
	model "LAPP_GAAS_GFrame_BACKEND/models/log"
	"LAPP_GAAS_GFrame_BACKEND/web/models"
	"github.com/go-xorm/xorm"
	"time"
)

var ConditionOfPackTemplatesOfArticle = grmi.NewCondition(
	map[string]grmi.ConditionItem{},
	nil,
)

var ConditionOfItemsOfPackOrder = grmi.NewCondition(
	map[string]grmi.ConditionItem{
		meta.PackOrderItemlst_PackOrderId.UrlParameterName: grmi.NewConditionItem(meta.PackOrderItemlst_PackOrderId, grmi.Equal, true),
	},
	nil,
)

var ConditionOfItemsOfPackOrderTemplate = grmi.NewCondition(
	map[string]grmi.ConditionItem{
		meta.PackTemplateItemlst_PackTemplateId.UrlParameterName: grmi.NewConditionItem(meta.PackTemplateItemlst_PackTemplateId, grmi.Equal, true),
	},
	nil,
)

type ExtServiceImplement struct {
}

func (self *ExtServiceImplement) Test(_ *models.Usertab) (string, error) {
	return "Aha!", nil
}

func (self *ExtServiceImplement) ClosePackOrder(user *models.Usertab, packOrderId string, poses []int) error {

	if _, err := db.Eloquent.Master().Transaction(func(session *xorm.Session) (interface{}, error) {

		packOrderDAO := dal.NewPackOrderDAO(session, user.Pid, user.Userid)
		order, err := packOrderDAO.SelectOne(packOrderId)
		if err != nil {
			return nil, err
		}
		if order == nil {
			return nil, grmi.NewBusinessError("不存在指定记录!")
		}
		if order.Status != 26 {
			return nil, grmi.NewBusinessError("包装单尚未激活!")
		}
		order.Status = 80
		packOrderItemlstDAO := dal.NewPackOrderItemlstDAO(session, user.Pid, user.Userid)
		predicates, err := ConditionOfItemsOfPackOrder.BuildPredicates(map[string]string{
			meta.PackOrderItemlst_PackOrderId.UrlParameterName: order.PackOrderId,
		})
		if err != nil {
			return nil, err
		}
		closedItems := make([]model.PackOrderItemlst, 0, 10)
		items, err := packOrderItemlstDAO.Select(predicates, ConditionOfPackTemplatesOfArticle.OrderByFields)
		if err != nil {
			return nil, err
		}
		if poses == nil || len(poses) <= 0 {
			for _, item := range items {
				closedItems = append(closedItems, item)
			}
		} else {
			posMapping := make(map[int]int, len(poses))
			for _, pos := range poses {
				posMapping[pos] = pos
			}
			for _, item := range items {
				if _, ok := posMapping[item.Pos]; ok {
					closedItems = append(closedItems, item)
				}
			}
		}
		for i := 0; i < len(closedItems); i++ {
			closedItems[i].Status = 80
		}
		err = packOrderDAO.UpdateOne(order)
		if err != nil {
			return nil, err
		}
		for i := 0; i < len(closedItems); i++ {
			err = packOrderItemlstDAO.UpdateOne(&closedItems[i])
			if err != nil {
				return nil, err
			}
		}
		if err != nil {
			return nil, err
		}
		return nil, nil
	}); err != nil {
		return err
	}
	return nil
}

func (self *ExtServiceImplement) OpenPackOrder(user *models.Usertab, artId string) ([]model.PackTemplate, error) {

	if result, err := db.Eloquent.Master().Transaction(func(session *xorm.Session) (interface{}, error) {

		articleDAO := base.NewArticleDAO(session, user.Pid, user.Userid)
		article, err := articleDAO.SelectOne(artId)
		if err != nil {
			return nil, err
		}
		if article == nil {
			return nil, grmi.NewBusinessError("没有在数据库中找到扫码内容对应的信息!")
		}
		packTemplateDAO := dal.NewPackTemplateDAO(session, user.Pid, user.Userid)
		predicates, err := ConditionOfItemsOfPackOrderTemplate.BuildPredicates(map[string]string{})
		if err != nil {
			return nil, err
		}
		packTemplates, err := packTemplateDAO.Select(predicates, ConditionOfPackTemplatesOfArticle.OrderByFields)
		if err != nil {
			return nil, err
		}
		if len(packTemplates) > 1 {
			return packTemplates, nil
		}
		if len(packTemplates) > 0 {
			err = self.CreatePackOrderWithTemplate(session, user, &packTemplates[0])
			if err != nil {
				return nil, err
			}
			return packTemplates, nil
		} else {
			return nil, grmi.NewBusinessError("没找到关联的包装单模板!")
		}
	}); err != nil {
		return nil, err
	} else {
		templates, ok := result.([]model.PackTemplate)
		if !ok {
			return nil, grmi.NewBusinessError("意外的返回结果!")
		}
		return templates, nil
	}
}

func (self *ExtServiceImplement) OpenPackOrderWithTemplate(user *models.Usertab, artId string, packTemplateId string) error {

	if _, err := db.Eloquent.Master().Transaction(func(session *xorm.Session) (interface{}, error) {

		articleDAO := base.NewArticleDAO(session, user.Pid, user.Userid)
		article, err := articleDAO.SelectOne(artId)
		if err != nil {
			return nil, err
		}
		if article == nil {
			return nil, grmi.NewBusinessError("没有在数据库中找到扫码内容对应的信息!")
		}
		packTemplateDAO := dal.NewPackTemplateDAO(session, user.Pid, user.Userid)
		packTemplate, err := packTemplateDAO.SelectOne(packTemplateId)
		if err != nil {
			return nil, err
		}
		if packTemplate == nil {
			return nil, grmi.NewBusinessError("不存在指定包装单模板!")
		}
		err = self.CreatePackOrderWithTemplate(session, user, packTemplate)
		if err != nil {
			return nil, err
		}
		return nil, nil
	}); err != nil {
		return err
	}
	return nil
}

func (self *ExtServiceImplement) CreatePackOrderWithTemplate(session *xorm.Session, user *models.Usertab, packTemplate *model.PackTemplate) error {

	packTemplateItemlstDAO := dal.NewPackTemplateItemlstDAO(session, user.Pid, user.Userid)
	predicates, err := ConditionOfItemsOfPackOrderTemplate.BuildPredicates(map[string]string{
		meta.PackTemplateItemlst_PackTemplateId.UrlParameterName: packTemplate.PackTemplateId,
	})
	if err != nil {
		return err
	}
	packTemplateItems, err := packTemplateItemlstDAO.Select(predicates, ConditionOfPackTemplatesOfArticle.OrderByFields)
	if err != nil {
		return err
	}
	packOrderId := "PACK" + time.Now().Format("20060102150405")
	packOrderDAO := dal.NewPackOrderDAO(session, user.Pid, user.Userid)
	err = packOrderDAO.InsertOne(&model.PackOrder{
		PlantNr:        packTemplate.PlantNr,
		PackOrderId:    packOrderId,
		Status:         26,
		PackTemplateId: packTemplate.PackTemplateId,
		PackTypeId:     packTemplate.PackTypeId,
		MaterialId:     packTemplate.MaterialId,
		PlanQty:        float64(packTemplate.PlanQty),
	})
	if err != nil {
		return err
	}
	items := make([]*model.PackOrderItemlst, 0, 10)
	for _, packTemplateItem := range packTemplateItems {
		items = append(items, &model.PackOrderItemlst{
			PlantNr:     packTemplateItem.PlantNr,
			PackOrderId: packOrderId,
			Pos:         packTemplateItem.Pos,
			PosCode:     packTemplateItem.PosCode,
			MaterialId:  packTemplateItem.MaterialId,
			PlanQty:     float64(packTemplateItem.PlanQty),
			Status:      26,
			Seq:         packTemplateItem.Seq,
		})
	}
	packOrderItemlstDAO := dal.NewPackOrderItemlstDAO(session, user.Pid, user.Userid)
	for _, item := range items {
		err = packOrderItemlstDAO.InsertOne(item)
		if err != nil {
			return err
		}
	}
	return nil
}
